package com.example.playertime;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import net.fabricmc.loader.api.FabricLoader;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

// 这个是gemini写的 (保留原注释，但代码已根据需求调整)
public class LocalizationManager {
    private final Map<String, String> translations = new HashMap<>();
    private final String defaultLanguage = "zh_cn"; // 默认语言代码

    public LocalizationManager(String languageCode) {
        // 尝试加载用户配置的语言
        loadLanguage(languageCode, false);
        // 如果配置的语言不是默认语言，加载默认语言作为备用
        if (!languageCode.equalsIgnoreCase(defaultLanguage)) {
            loadLanguage(defaultLanguage, true);
        }
        // 如果配置的语言和默认语言都没加载成功，或者配置的语言文件不完整，确保默认语言至少被加载
        if (translations.isEmpty() && !languageCode.equalsIgnoreCase(defaultLanguage)) {
            PlayerTimeMod.LOGGER.warn("[PlayerTime] 未能加载配置的语言文件({})或默认语言文件({})，Mod将使用原始键。", languageCode, defaultLanguage);
        } else if (translations.isEmpty() && languageCode.equalsIgnoreCase(defaultLanguage)) {
            PlayerTimeMod.LOGGER.warn("[PlayerTime] 未能加载默认语言文件({})，Mod将使用原始键。", defaultLanguage);
        }
    }

    private void loadLanguage(String languageCode, boolean isFallback) {
        String resourcePath = String.format("assets/playertime/lang/%s.json", languageCode.toLowerCase()); // 确保小写
        try {
            // 使用 FabricLoader 获取 Mod 资源
            InputStream is = FabricLoader.getInstance().getModContainer("playertime")
                    .flatMap(container -> container.findPath(resourcePath))
                    .map(path -> {
                        try {
                            return path.toUri().toURL().openStream();
                        } catch (Exception e) {
                            // 查找路径或打开流失败
                            return null;
                        }
                    })
                    .orElse(null); // 如果找不到路径或打开失败，返回 null

            if (is == null) {
                if (!isFallback) {
                    PlayerTimeMod.LOGGER.warn("[PlayerTime] 未找到语言文件: {}", resourcePath);
                }
                return;
            }

            try (InputStreamReader reader = new InputStreamReader(is, StandardCharsets.UTF_8)) {
                JsonObject json = JsonParser.parseReader(reader).getAsJsonObject();
                json.entrySet().forEach(entry -> {
                    // 如果是加载备用语言，只有当主语言没有该键时才添加
                    // 如果是加载主语言，则覆盖备用语言中的同名键
                    if (!translations.containsKey(entry.getKey()) || !isFallback) {
                        translations.put(entry.getKey(), entry.getValue().getAsString());
                    }
                });
                PlayerTimeMod.LOGGER.info("[PlayerTime] 已加载 {} 语言字符串 ({}): {}", translations.size(), isFallback ? "备用" : "主要", languageCode);
            }

        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[PlayerTime] 加载语言文件失败 ({}): {}", languageCode, resourcePath, e);
        }
    }

    /**
     * 根据键获取本地化字符串。如果找不到键，返回键本身。
     * @param key 本地化键
     * @return 本地化字符串或原始键
     */
    public String getString(String key) {
        return translations.getOrDefault(key, key);
    }

    /**
     * 根据键获取本地化字符串，并使用参数格式化。如果找不到键，返回原始键。
     * @param key 本地化键
     * @param args 格式化参数
     * @return 格式化后的本地化字符串或原始键
     */
    public String getString(String key, Object... args) {
        String pattern = getString(key);
        try {
            // MessageFormat 可以处理 {0}, {1} 等占位符
            return MessageFormat.format(pattern, args);
        } catch (IllegalArgumentException e) {
            PlayerTimeMod.LOGGER.error("[PlayerTime] 格式化本地化字符串失败 (键: {})", key, e);
            return pattern; // 格式化失败时返回原始模式
        }
    }

    /**
     * 获取所有本地化字符串的映射。主要用于 Web API。
     * @return 包含所有本地化键值对的 Map
     */
    public Map<String, String> getAllStrings() {
        return new HashMap<>(translations); // 返回副本，防止外部修改
    }
}
