package com.example.playertime;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayConnectionEvents;
import net.fabricmc.loader.api.FabricLoader;
import net.fabricmc.loader.api.ModContainer;
import net.fabricmc.loader.api.Version;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.ClickEvent;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;
import net.minecraft.util.Formatting;
import net.minecraft.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class PlayerTimeMod implements ModInitializer {
    public static final Logger LOGGER = LoggerFactory.getLogger("PlayerTimeTracker");
    private static PlayerTimeTracker timeTracker;
    private static WebServer webServer;
    private static ModConfig config;
    public static LocalizationManager localizationManager;

    private static final String RSS_FEED_URL = "https://git.branulf.top/Branulf/ServerPlayerOnlineTracker/releases.rss";
    private static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(2); // One for update check, one for saving

    @Override
    public void onInitialize() {
        try {
            LOGGER.info("[在线时间] 初始化 玩家在线时长视奸Mod...");

            // 加载配置
            config = new ModConfig(FabricLoader.getInstance().getConfigDir());
            // 初始化本地化管理器
            localizationManager = new LocalizationManager(config.getLanguage());

            // 检查更新
            checkForUpdates();

            // 服务器启动中事件
            ServerLifecycleEvents.SERVER_STARTING.register(server -> {
                LOGGER.info("[在线时间] 服务器启动中...");
                timeTracker = new PlayerTimeTracker(server, config);
                try {
                    webServer = new WebServer(timeTracker, config.getWebPort(), server);
                    webServer.start();
                    LOGGER.info("[在线时间] Web服务器在端口 {} 启动", config.getWebPort());
                } catch (Exception e) {
                    LOGGER.error("[在线时间] 无法启动Web服务器", e);
                }
            });

            // 服务器已启动事件
            ServerLifecycleEvents.SERVER_STARTED.register(server -> {
                LOGGER.info("[在线时间] 服务器已启动.");
                if (timeTracker != null) {
                    timeTracker.loadData();
                    // 启动定时保存任务
                    int saveInterval = config.getSaveIntervalMinutes();
                    if (saveInterval > 0) {
                        scheduler.scheduleAtFixedRate(timeTracker::saveAll, saveInterval, saveInterval, TimeUnit.MINUTES);
                        LOGGER.info("[在线时间] 已安排每 {} 分钟定时保存玩家数据", saveInterval);
                    } else {
                        LOGGER.warn("[在线时间] 配置的保存间隔为 {} 分钟，定时保存已禁用。", saveInterval);
                    }
                } else {
                    LOGGER.error("[在线时间] PlayerTimeTracker 未在 SERVER_STARTING 阶段成功初始化!");
                }
            });

            // 玩家加入事件
            ServerPlayConnectionEvents.JOIN.register((handler, sender, server) -> {
                if (timeTracker != null) {
                    // 根据配置决定是否跟踪该玩家
                    if (!config.isWhitelistOnly() || server.getPlayerManager().isWhitelisted(handler.player.getGameProfile())) {
                        timeTracker.onPlayerJoin(handler.player);
                    } else {
                        LOGGER.debug("[在线时间] 玩家 {} ({}) 不在白名单，且配置为仅记录白名单玩家，跳过跟踪。", handler.player.getName().getString(), handler.player.getUuid());
                    }
                }
            });

            // 玩家离开事件
            ServerPlayConnectionEvents.DISCONNECT.register((handler, server) -> {
                if (timeTracker != null) {
                    if (timeTracker.isPlayerTracked(handler.player.getUuid())) {
                        timeTracker.onPlayerLeave(handler.player);
                    } else {
                        LOGGER.debug("[在线时间] 玩家 {} ({}) 未被跟踪，跳过离开事件处理。", handler.player.getName().getString(), handler.player.getUuid());
                    }
                }
            });

            // 服务器停止事件
            ServerLifecycleEvents.SERVER_STOPPING.register(server -> {
                LOGGER.info("[在线时间] 服务器停止中 - 正在保存数据...");
                if (timeTracker != null) {
                    for (ServerPlayerEntity player : server.getPlayerManager().getPlayerList()) {
                        if (timeTracker.isPlayerTracked(player.getUuid())) {
                            timeTracker.onPlayerLeave(player);
                        }
                    }
                    timeTracker.saveAll();
                }

                if (webServer != null) {
                    webServer.stop();
                    LOGGER.info("[在线时间] Web服务器已停止.");
                }

                scheduler.shutdownNow();
                LOGGER.info("[在线时间] 定时任务已关闭.");

                LOGGER.info("[在线时间] 数据保存完成，Mod 已停止.");
            });

        } catch (Exception e) {
            LOGGER.error("[在线时间] Mod初始化失败", e);
            throw new RuntimeException("[在线时间] Mod初始化失败 ", e);
        }

        registerCommands();
    }

    public static ModConfig getConfig() {
        return config;
    }

    public static PlayerTimeTracker getTimeTracker() {
        return timeTracker;
    }

    public static LocalizationManager getLocalizationManager() {
        return localizationManager;
    }

    // 注册命令
    private void registerCommands() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("onlineTime")
                    .requires(source -> source.hasPermissionLevel(0)) // 任何玩家都可以使用
                    .executes(context -> showOnlineTime(context.getSource(), 1)) // 默认显示第一页
                    .then(CommandManager.argument("page", IntegerArgumentType.integer(1))
                            .executes(context -> showOnlineTime(
                                    context.getSource(),
                                    IntegerArgumentType.getInteger(context, "page")
                            ))
                    )
            );
            LOGGER.info("[在线时间] 已注册命令 /onlineTime");
        });
    }

    // 显示在线时间
    private int showOnlineTime(ServerCommandSource source, int requestedPage) {
        ServerPlayerEntity player = source.getPlayer();
        if (player == null) {
            source.sendMessage(Text.literal(localizationManager.getString("playertime.command.error.player_only")).formatted(Formatting.RED));
            return 0;
        }

        Util.getMainWorkerExecutor().execute(() -> {
            PlayerTimeTracker tracker = getTimeTracker();
            if (tracker == null) {
                player.sendMessage(Text.literal(localizationManager.getString("playertime.command.error.not_initialized")).formatted(Formatting.RED), false);
                return;
            }

            List<PlayerTimeTracker.PlayerTimeStatsWithNames> statsList = tracker.getSortedPlayerStats();

            sendPaginatedMessage(player, statsList, requestedPage);
        });

        return 1;
    }

    // 发送分页消息
    private void sendPaginatedMessage(ServerPlayerEntity player, List<PlayerTimeTracker.PlayerTimeStatsWithNames> statsList, int requestedPage) {
        int pageSize = 10;
        int totalEntries = statsList.size();
        int totalPages = (totalEntries + pageSize - 1) / pageSize;
        int currentPage = Math.max(1, Math.min(requestedPage, Math.max(1, totalPages)));

        int fromIndex = (currentPage - 1) * pageSize;
        int toIndex = Math.min(fromIndex + pageSize, totalEntries);

        player.sendMessage(Text.literal(localizationManager.getString("playertime.command.title", currentPage, Math.max(1, totalPages))).formatted(Formatting.GOLD), false);

        if (statsList.isEmpty()) {
            player.sendMessage(Text.literal(localizationManager.getString("playertime.command.empty_stats")).formatted(Formatting.GRAY), false);
            return;
        }

        for (int i = fromIndex; i < toIndex; i++) {
            PlayerTimeTracker.PlayerTimeStatsWithNames stats = statsList.get(i);
            String formattedStats = localizationManager.getString(
                    "playertime.stats.format",
                    PlayerTimeTracker.formatTime(stats.getTotalTime()),
                    PlayerTimeTracker.formatTime(stats.last30Days),
                    PlayerTimeTracker.formatTime(stats.last7Days)
            );
            player.sendMessage(Text.literal(String.format("§e%s§r: %s", stats.playerName, formattedStats)), false);
        }

        MutableText footer = Text.literal("");
        if (currentPage > 1) {
            footer.append(Text.literal(localizationManager.getString("playertime.command.prev_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (currentPage - 1)
                    )).withColor(Formatting.GREEN)));
        }

        if (currentPage > 1 && currentPage < totalPages) {
            footer.append(" ");
        }
        footer.append(Text.literal(localizationManager.getString("playertime.command.total_players", totalEntries)).formatted(Formatting.GRAY));


        if (currentPage < totalPages) {
            if (currentPage > 1 || totalEntries > 0) {
                footer.append(" ");
            }
            footer.append(Text.literal(localizationManager.getString("playertime.command.next_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (currentPage + 1)
                    )).withColor(Formatting.GREEN)));
        }

        player.sendMessage(footer, false);
    }


    // 检查更新
    private void checkForUpdates() {
        scheduler.submit(() -> {
            try {
                Optional<ModContainer> modContainer = FabricLoader.getInstance().getModContainer("playertime");
                if (modContainer.isEmpty()) {
                    LOGGER.warn("[在线时间] 无法获取 Mod 容器，跳过更新检查。");
                    return;
                }
                Version currentVersion = modContainer.get().getMetadata().getVersion();
                String currentVersionString = currentVersion.getFriendlyString();

                LOGGER.info("[在线时间] 正在检查更新...");

                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = factory.newDocumentBuilder();
                Document doc = builder.parse(new URL(RSS_FEED_URL).openStream());
                doc.getDocumentElement().normalize();

                NodeList itemList = doc.getElementsByTagName("item");

                if (itemList.getLength() == 0) {
                    LOGGER.warn("[在线时间] RSS Feed 中没有找到任何发布项。");
                    return;
                }

                Element latestItem = (Element) itemList.item(0);
                String latestVersionString = latestItem.getElementsByTagName("title").item(0).getTextContent();
                String latestVersionLink = latestItem.getElementsByTagName("link").item(0).getTextContent();

                LOGGER.info("[在线时间] 当前版本: {}, 最新版本 (RSS): {}", currentVersionString, latestVersionString);

                if (isNewerVersion(latestVersionString, currentVersionString)) {
                    LOGGER.warn("==================================================");
                    LOGGER.warn("[在线时间] 发现新版本!");
                    LOGGER.warn("[在线时间] 当前版本: {}", currentVersionString);
                    LOGGER.warn("[在线时间] 最新版本: {}", latestVersionString);
                    LOGGER.warn("[在线时间] 下载链接: {}", latestVersionLink);
                    LOGGER.warn("==================================================");
                } else {
                    LOGGER.info("[在线时间] 当前已是最新版本(或检查失败)。");
                }

            } catch (javax.xml.parsers.ParserConfigurationException e) {
                LOGGER.error("[在线时间] 更新检查失败: XML 解析器配置错误", e);
            } catch (org.xml.sax.SAXException e) {
                LOGGER.error("[在线时间] 更新检查失败: XML 解析错误", e);
            } catch (java.io.IOException e) {
                LOGGER.error("[在线时间] 更新检查失败: 网络或文件读取错误", e);
            } catch (Exception e) {
                LOGGER.error("[在线时间] 更新检查时发生未知错误", e);
            }
        });
    }


    // 检查版本号是否比当前版本高
    private boolean isNewerVersion(String newVersion, String currentVersion) {
        if (newVersion == null || currentVersion == null || newVersion.isEmpty() || currentVersion.isEmpty()) {
            return false;
        }

        String cleanNewVersion = newVersion.toLowerCase().startsWith("v") ? newVersion.substring(1) : newVersion;
        String cleanCurrentVersion = currentVersion.toLowerCase().startsWith("v") ? currentVersion.substring(1) : currentVersion;

        String[] newParts = cleanNewVersion.split("\\.");
        String[] currentParts = cleanCurrentVersion.split("\\.");

        int maxLength = Math.max(newParts.length, currentParts.length);

        for (int i = 0; i < maxLength; i++) {
            int newPart = (i < newParts.length) ? parseIntOrZero(newParts[i]) : 0;
            int currentPart = (i < currentParts.length) ? parseIntOrZero(currentParts[i]) : 0;

            if (newPart > currentPart) {
                return true;
            }
            if (newPart < currentPart) {
                return false;
            }
        }

        return false;
    }

    // 尝试将字符串转换为整数，如果转换失败则返回0
    private int parseIntOrZero(String s) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {
            return 0;
        }
    }
}
