package com.example.playertime;

import com.google.gson.*;
import com.mojang.authlib.GameProfile;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.PlayerManager;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.util.Uuids;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

public class PlayerTimeTracker {
    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();
    private final MinecraftServer server;
    private final Path dataFile;
    private final Map<UUID, PlayerTimeData> playerData = new ConcurrentHashMap<>();
    private final ModConfig config;

    public PlayerTimeTracker(MinecraftServer server, ModConfig config) {
        this.server = server;
        this.config = config;
        this.dataFile = server.getRunDirectory().resolve("player_time_data.json");
    }

    public void loadData() {
        if (!Files.exists(dataFile)) {
            PlayerTimeMod.LOGGER.info("[在线时间] 数据文件未找到 ({})，跳过加载。", dataFile);
            return;
        }

        try (Reader reader = Files.newBufferedReader(dataFile, StandardCharsets.UTF_8)) {
            JsonElement jsonElement = JsonParser.parseReader(reader);

            if (jsonElement == null || !jsonElement.isJsonObject()) {
                PlayerTimeMod.LOGGER.warn("[在线时间] 数据文件 ({}) 为空或格式错误，跳过加载。", dataFile);
                return;
            }

            JsonObject root = jsonElement.getAsJsonObject();
            int resetCount = 0;
            int loadedCount = 0;

            for (Map.Entry<String, JsonElement> entry : root.entrySet()) {
                try {
                    UUID uuid = UUID.fromString(entry.getKey());
                    PlayerTimeData data = GSON.fromJson(entry.getValue(), PlayerTimeData.class);

                    // 检查并重置上次登录时间，防止服务器崩溃导致时间计算错误
                    if (data.lastLogin > 0) {
                        PlayerTimeMod.LOGGER.warn(
                                "[在线时间] 在数据加载过程中发现玩家（UUID：{}）的最后登录时间大于0（{}）。将其重置为0。",
                                uuid, data.lastLogin
                        );
                        data.lastLogin = 0;
                        resetCount++;
                    }

                    playerData.put(uuid, data);
                    loadedCount++;
                } catch (IllegalArgumentException e) {
                    PlayerTimeMod.LOGGER.error("[在线时间] 数据文件中的 UUID 格式无效: " + entry.getKey(), e);
                } catch (JsonParseException e) {
                    PlayerTimeMod.LOGGER.error("[在线时间] 解析玩家数据失败(UUID: " + entry.getKey() + ")", e);
                }
            }
            PlayerTimeMod.LOGGER.info("[在线时间] 成功加载了 {} 名玩家的数据，重置了 {} 名玩家的上次登录时间。", loadedCount, resetCount);

        } catch (IOException e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 无法读取玩家在线时间数据文件 ({})", dataFile, e);
        } catch (JsonParseException e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 玩家在线时间数据文件 ({}) 格式错误", dataFile, e);
        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 加载玩家在线时间数据时发生未知错误", e);
        }
    }

    public void onPlayerJoin(ServerPlayerEntity player) {
        UUID uuid = player.getUuid();
        // 检查配置是否白名单
        if (config.isWhitelistOnly() && !server.getPlayerManager().isWhitelisted(player.getGameProfile())) {
            PlayerTimeMod.LOGGER.debug("[在线时间] 玩家 {} ({}) 不在白名单，且配置为仅记录白名单玩家，跳过加入处理。", player.getName().getString(), uuid);
            return;
        }

        PlayerTimeData data = playerData.computeIfAbsent(uuid, k -> {
            PlayerTimeMod.LOGGER.info("[在线时间] 为新玩家 {} ({}) 创建数据。", player.getName().getString(), uuid);
            return new PlayerTimeData();
        });

        // 记录本次登录的开始时间
        data.lastLogin = Instant.now().getEpochSecond();
        PlayerTimeMod.LOGGER.info("[在线时间] 玩家 {} ({}) 加入, 开始计时。", player.getName().getString(), uuid);
    }

    public void onPlayerLeave(ServerPlayerEntity player) {
        UUID uuid = player.getUuid();
        PlayerTimeData data = playerData.get(uuid);

        // 防幽灵
        if (data != null && data.lastLogin > 0) {
            long now = Instant.now().getEpochSecond();
            long sessionTime = now - data.lastLogin;

            if (sessionTime > 0) {
                data.totalTime += sessionTime;
                data.rolling30Days.addPlayTime(now, sessionTime);
                data.rolling7Days.addPlayTime(now, sessionTime);

                PlayerTimeMod.LOGGER.info("[在线时间] 玩家 {} ({}) 离开, 会话时间 {} 秒, 总计时间 {} 秒。",
                        player.getName().getString(), uuid, sessionTime, data.totalTime);
            } else {
                // 防其他因素
                PlayerTimeMod.LOGGER.warn("[在线时间] 玩家 {} ({}) 离开, 但计算出的会话时间不正常 ({}), 跳过时间累加。",
                        player.getName().getString(), uuid, sessionTime);
            }

            // 标记离线
            data.lastLogin = 0;
        } else {
            PlayerTimeMod.LOGGER.debug("[在线时间] 玩家 {} ({}) 离开，但数据不存在或上次登录时间为0，跳过会话时间计算。", player.getName().getString(), uuid);
        }
    }

    public boolean isPlayerTracked(UUID uuid) {
        return playerData.containsKey(uuid);
    }


    public List<PlayerTimeStatsWithNames> getSortedPlayerStats() {
        long now = Instant.now().getEpochSecond();
        List<PlayerTimeStatsWithNames> statsList = new ArrayList<>();

        // 获取白名单 UUID 集合 (如果需要过滤)
        Set<UUID> whitelistUuids;
        if (config.isWhitelistOnly()) {
            whitelistUuids = new HashSet<>();
            if (server != null && server.getPlayerManager() != null && server.getUserCache() != null) {
                for (String name : server.getPlayerManager().getWhitelist().getNames()) {
                    server.getUserCache().findByName(name).ifPresent(profile -> {
                        whitelistUuids.add(profile.getId());
                    });
                }
            } else {
                PlayerTimeMod.LOGGER.warn("[在线时间] UserCache 或 PlayerManager 不可用，无法获取白名单列表进行过滤!");
                // 异常处理，返回空列表
                return Collections.emptyList();
            }
        } else {
            whitelistUuids = null;
        }

        for (Map.Entry<UUID, PlayerTimeData> entry : playerData.entrySet()) {
            UUID uuid = entry.getKey();
            PlayerTimeData data = entry.getValue();

            // 是不是白名单玩家
            if (config.isWhitelistOnly() && (whitelistUuids == null || !whitelistUuids.contains(uuid))) {
                continue;
            }

            // 计算当前总时间
            long currentTotalTime = data.totalTime;
            // 只有当玩家在线时才计算当前会话时间
            if (data.lastLogin > 0 && server.getPlayerManager().getPlayer(uuid) != null) {
                long currentSessionTime = now - data.lastLogin;
                if (currentSessionTime > 0) {
                    currentTotalTime += currentSessionTime;
                }
            }

            // 获取滚动时间
            long last30DaysTime = data.rolling30Days.getTotalTime(now);
            long last7DaysTime = data.rolling7Days.getTotalTime(now);

            // 获取玩家名称
            String playerName = getPlayerName(uuid);

            statsList.add(new PlayerTimeStatsWithNames(uuid, playerName, currentTotalTime, last30DaysTime, last7DaysTime));
        }

        // 按总时排序
        statsList.sort(Comparator.comparingLong(PlayerTimeStatsWithNames::getTotalTime).reversed());

        return statsList;
    }

    public String getPlayerName(UUID uuid) {
        // 尝试获取在线玩家名称
        ServerPlayerEntity player = server.getPlayerManager().getPlayer(uuid);
        if (player != null) {
            return player.getName().getString();
        }

        // 尝试从 UserCache 获取名称，主要用于离线服务器
        if (server != null && server.getUserCache() != null) {
            Optional<GameProfile> profile = server.getUserCache().getByUuid(uuid);
            if (profile.isPresent()) {
                return profile.get().getName();
            }
        } else {
            PlayerTimeMod.LOGGER.debug("[在线时间] 尝试通过 UUID 获取玩家名称时 UserCache 不可用: {}", uuid);
        }

        return uuid.toString();
    }

    public void saveAll() {
        PlayerTimeMod.LOGGER.info("[在线时间] 开始保存所有玩家数据...");
        long now = Instant.now().getEpochSecond();
        JsonObject root = new JsonObject();
        int savedCount = 0;

        for (ServerPlayerEntity player : server.getPlayerManager().getPlayerList()) {
            UUID uuid = player.getUuid();
            PlayerTimeData data = playerData.get(uuid);
            // 只有被跟踪且没标记离线
            if (data != null && data.lastLogin > 0) {
                long sessionTime = now - data.lastLogin;
                if (sessionTime > 0) {
                    data.totalTime += sessionTime;
                    data.rolling30Days.addPlayTime(now, sessionTime);
                    data.rolling7Days.addPlayTime(now, sessionTime);
                    // 更新 lastLogin 到当前时间，防止炸鱼
                    data.lastLogin = now;
                    PlayerTimeMod.LOGGER.debug("[在线时间] 定时保存：更新玩家 {} ({}) 会话时间 {} 秒。", player.getName().getString(), uuid, sessionTime);
                } else {
                    PlayerTimeMod.LOGGER.debug("[在线时间] 定时保存：玩家 {} ({}) 会话时间不正常 ({})，跳过更新。", player.getName().getString(), uuid, sessionTime);
                }
            }
        }

        // 遍历所有玩家数据进行保存
        for (Map.Entry<UUID, PlayerTimeData> entry : playerData.entrySet()) {
            UUID uuid = entry.getKey();
            PlayerTimeData data = entry.getValue();

            boolean isWhitelisted = false;
            if (server != null && server.getPlayerManager() != null) {
                // 优先检查在线玩家
                ServerPlayerEntity onlinePlayer = server.getPlayerManager().getPlayer(uuid);
                if (onlinePlayer != null) {
                    isWhitelisted = server.getPlayerManager().isWhitelisted(onlinePlayer.getGameProfile());
                } else if (server.getUserCache() != null) {
                    // 检查离线玩家是否在白名单
                    Optional<GameProfile> profile = server.getUserCache().getByUuid(uuid);
                    if (profile.isPresent()) {
                        isWhitelisted = server.getPlayerManager().isWhitelisted(profile.get());
                    }
                }
            } else {
                PlayerTimeMod.LOGGER.warn("[在线时间] UserCache 或 PlayerManager 不可用，无法在保存时检查白名单状态。");
                // 如果无法检查白名单，且配置为只记录白名单，则跳过保存此玩家数据
                if (config.isWhitelistOnly()) {
                    PlayerTimeMod.LOGGER.debug("[在线时间] 无法检查白名单状态，且配置为仅记录白名单，跳过保存玩家 {} ({}) 的数据。", getPlayerName(uuid), uuid);
                    continue;
                }
            }


            if (config.isWhitelistOnly() && !isWhitelisted) {
                PlayerTimeMod.LOGGER.debug("[在线时间] 配置为仅记录白名单，跳过保存非白名单玩家 {} ({}) 的数据。", getPlayerName(uuid), uuid);
                continue;
            }

            root.add(uuid.toString(), GSON.toJsonTree(data));
            savedCount++;
        }


        try {
            Path dataDir = dataFile.getParent();

            if (dataDir != null) {
                Files.createDirectories(dataDir);
            } else {
                PlayerTimeMod.LOGGER.debug("[在线时间] 数据文件 {} 位于根目录，跳过创建父目录。", dataFile);
            }

            Path tempFile = dataFile.resolveSibling(dataFile.getFileName().toString() + ".tmp");

            try (Writer writer = Files.newBufferedWriter(tempFile, StandardCharsets.UTF_8)) {
                GSON.toJson(root, writer);
            }
            Files.move(tempFile, dataFile, java.nio.file.StandardCopyOption.REPLACE_EXISTING);

            PlayerTimeMod.LOGGER.info("[在线时间] 所有玩家数据 ({}) 已成功保存，共 {} 条记录。", dataFile, savedCount);
        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 无法保存所有玩家在线时间数据到 {}", dataFile, e);
        }
    }

    public static String formatTime(long seconds) {
        if (seconds < 0) seconds = 0;
        long hours = seconds / 3600;
        long minutes = (seconds % 3600) / 60;
        return String.format("%dh %02dm", hours, minutes);
    }

    public static class PlayerTimeData {
        long totalTime = 0; // 总在线时间
        long lastLogin = 0; // 上次登录时间戳
        RollingTimeWindow rolling30Days = new RollingTimeWindow(30); // 最近30天
        RollingTimeWindow rolling7Days = new RollingTimeWindow(7);   // 最近7天
    }

    private static class RollingTimeWindow {
        private final int days;
        private final List<TimeEntry> entries = new LinkedList<>();

        @SuppressWarnings("unused")
        private RollingTimeWindow() {
            this.days = 0;
        }

        public RollingTimeWindow(int days) {
            this.days = days;
        }


        public void addPlayTime(long timestamp, long seconds) {
            if (seconds <= 0) return;
            entries.add(new TimeEntry(timestamp, seconds));
            cleanUp(timestamp);
        }


        public long getTotalTime(long currentTime) {
            cleanUp(currentTime);
            return entries.stream().mapToLong(e -> e.seconds).sum();
        }


        private void cleanUp(long currentTime) {
            long cutoff = currentTime - (days * 24 * 3600L);
            entries.removeIf(entry -> entry.timestamp < cutoff);
        }


        private static class TimeEntry {
            final long timestamp;
            final long seconds;

            // Gson 需要一个无参构造函数
            @SuppressWarnings("unused")
            private TimeEntry() {
                this.timestamp = 0;
                this.seconds = 0;
            }

            TimeEntry(long timestamp, long seconds) {
                this.timestamp = timestamp;
                this.seconds = seconds;
            }
        }
    }


    public static class PlayerTimeStatsWithNames {
        public UUID uuid;
        public String playerName;
        private long totalTime;
        public long last30Days;
        public long last7Days;

        @SuppressWarnings("unused")
        private PlayerTimeStatsWithNames() {}

        public PlayerTimeStatsWithNames(UUID uuid, String playerName, long totalTime, long last30Days, long last7Days) {
            this.uuid = uuid;
            this.playerName = playerName;
            this.totalTime = totalTime;
            this.last30Days = last30Days;
            this.last7Days = last7Days;
        }

        public long getTotalTime() {
            return totalTime;
        }
    }

    public Map<UUID, PlayerTimeData> getPlayerData() {
        return Collections.unmodifiableMap(playerData);
    }

    public Path getDataFile() {
        return this.dataFile;
    }

    public Map<String, List<Map<String, String>>> getOnlinePlayersCategorized() {
        List<Map<String, String>> whitelisted = new ArrayList<>();
        List<Map<String, String>> nonWhitelisted = new ArrayList<>();

        PlayerManager playerManager = server.getPlayerManager();
        Set<UUID> whitelistUuids = new HashSet<>();

        if (server != null && server.getUserCache() != null) {
            for (String name : playerManager.getWhitelist().getNames()) {
                server.getUserCache().findByName(name).ifPresent(profile -> {
                    whitelistUuids.add(profile.getId());
                });
            }
        } else {
            PlayerTimeMod.LOGGER.warn("[在线时间] UserCache 不可用，无法获取白名单UUID用于在线玩家分类。");
        }

        for (ServerPlayerEntity player : playerManager.getPlayerList()) {
            Map<String, String> playerInfo = new HashMap<>();
            playerInfo.put("name", player.getName().getString());
            playerInfo.put("uuid", player.getUuid().toString());

            if (whitelistUuids.contains(player.getUuid())) {
                whitelisted.add(playerInfo);
            } else {
                nonWhitelisted.add(playerInfo);
            }
        }

        Map<String, List<Map<String, String>>> result = new HashMap<>();
        result.put("whitelisted", whitelisted);
        result.put("non_whitelisted", nonWhitelisted);
        return result;
    }

    public Map<String, Integer> getOnlinePlayerCounts() {
        int total = 0;
        int whitelisted = 0;
        int nonWhitelisted = 0;

        PlayerManager playerManager = server.getPlayerManager();
        Set<UUID> whitelistUuids = new HashSet<>();

        if (server != null && server.getUserCache() != null) {
            for (String name : playerManager.getWhitelist().getNames()) {
                server.getUserCache().findByName(name).ifPresent(profile -> {
                    whitelistUuids.add(profile.getId());
                });
            }
        } else {
            PlayerTimeMod.LOGGER.warn("[在线时间] UserCache 不可用，无法获取白名单UUID用于玩家计数分类。");
        }

        for (ServerPlayerEntity player : playerManager.getPlayerList()) {
            total++;
            if (whitelistUuids.contains(player.getUuid())) {
                whitelisted++;
            } else {
                nonWhitelisted++;
            }
        }

        Map<String, Integer> result = new HashMap<>();
        result.put("total", total);
        result.put("whitelisted", whitelisted);
        result.put("non_whitelisted", nonWhitelisted);
        return result;
    }

    public List<Map<String, Object>> getWhitelistPlayers() {
        List<Map<String, Object>> whitelist = new ArrayList<>();
        PlayerManager playerManager = server.getPlayerManager();

        if (server != null && server.getUserCache() != null) {
            for (String name : playerManager.getWhitelist().getNames()) {
                Map<String, Object> player = new HashMap<>();
                player.put("name", name);

                Optional<GameProfile> profile = server.getUserCache().findByName(name);
                if (profile.isPresent()) {
                    UUID uuid = profile.get().getId();
                    player.put("uuid", uuid.toString());
                    ServerPlayerEntity onlinePlayer = playerManager.getPlayer(uuid);
                    player.put("online", onlinePlayer != null);
                } else {
                    UUID offlineUuid = Uuids.getOfflinePlayerUuid(name);
                    player.put("uuid", offlineUuid.toString());
                    ServerPlayerEntity onlinePlayer = playerManager.getPlayer(offlineUuid);
                    player.put("online", onlinePlayer != null);
                }

                whitelist.add(player);
            }
        } else {
            PlayerTimeMod.LOGGER.warn("[在线时间] UserCache 不可用，无法获取白名单列表。");
        }

        return whitelist;
    }
}
