document.addEventListener('DOMContentLoaded', function() {
    // 初始化变量
    const tpsHistory = Array(60).fill(20);
    const msptHistory = Array(60).fill(0);
    const performanceLabels = Array(60).fill('');
    let memoryChart = null;
    let performanceChart = null;
    let diskChart = null;
    let lang = {};

    // DOM元素
    const elements = {
        refreshBtn: document.getElementById('refresh-btn'),
        themeToggle: document.getElementById('theme-toggle'),
        statsTableBody: document.getElementById('stats-table')?.getElementsByTagName('tbody')[0],
        statsTableHeader: document.getElementById('stats-table')?.getElementsByTagName('thead')[0],
        whitelistPlayers: document.getElementById('whitelist-players'),
        nonWhitelistPlayers: document.getElementById('non-whitelist-players'),
        totalCount: document.getElementById('total-count'),
        whitelistCount: document.getElementById('whitelist-count'),
        nonWhitelistCount: document.getElementById('non-whitelist-count'),
        memoryUsed: document.getElementById('memory-used'),
        memoryFree: document.getElementById('memory-free'),
        memoryPercent: document.getElementById('memory-percent'),
        diskTotal: document.getElementById('disk-total'),
        diskUsed: document.getElementById('disk-used'),
        diskFree: document.getElementById('disk-free'),
        diskPercent: document.getElementById('disk-percent'),
        processors: document.getElementById('processors'),
        serverVersion: document.getElementById('server-version'),
        serverPlayerCount: document.getElementById('server-player-count'),
        serverMaxPlayers: document.getElementById('server-max-players'),
        serverMotd: document.getElementById('server-motd'),
        uptime: document.getElementById('uptime'),
        tpsValue: document.getElementById('tps-value'),
        msptValue: document.getElementById('mspt-value'),
        floatingRefreshBtn: document.getElementById('floating-refresh-btn')
    };

    // 加载保存的主题
    const savedTheme = localStorage.getItem('theme') || 'light';
    document.documentElement.setAttribute('data-theme', savedTheme);

    // 初始设置：加载语言，初始化图表，加载数据，设置刷新间隔
    loadLanguage().then(() => {
        translatePage();
        initCharts();
        loadAllData();
        // 设置定期刷新（每10秒一次），后续加入配置文件
        const refreshInterval = setInterval(loadAllData, 10000);
        console.log('数据刷新间隔设置为10秒。');
    }).catch(error => {
        console.error('加载语言或初始数据失败:', error);
        showError('加载语言或初始数据失败。请检查控制台。');
    });

    if (elements.themeToggle) {
        elements.themeToggle.addEventListener('click', toggleTheme);
    }

    if (elements.refreshBtn) {
        elements.refreshBtn.addEventListener('click', handleRefresh);
    }

    if (elements.floatingRefreshBtn) {
        elements.floatingRefreshBtn.addEventListener('click', handleRefresh);
    }


    async function loadLanguage() {
        try {
            const response = await fetch('/api/lang');
            if (!response.ok) {
                const errorText = await response.text();
                console.error(`获取/api/lang时HTTP错误! 状态: ${response.status}, 内容: ${errorText}`);
                throw new Error(`HTTP错误! 状态: ${response.status}`);
            }
            lang = await response.json();
            console.log('语言文件加载成功。');
        } catch (error) {
            console.error('无法从/api/lang加载语言文件:', error);
            lang = {};
        }
    }

    function translatePage() {
        document.querySelectorAll('[data-lang-key]').forEach(element => {
            const key = element.getAttribute('data-lang-key');
            if (lang[key]) {
                if (element.tagName === 'TITLE') {
                    document.title = lang[key];
                } else {
                    element.textContent = lang[key];
                }
            } else {
            }
        });
        // 加载语言后更新图表标签
        if (memoryChart) {
            memoryChart.data.labels = [getLangString('playertime.web.chart.memory_used'), getLangString('playertime.web.chart.memory_free')];
            memoryChart.update();
        }
        if (performanceChart) {
            performanceChart.data.datasets[0].label = getLangString('playertime.web.chart.tps');
            performanceChart.data.datasets[1].label = getLangString('playertime.web.chart.mspt');
            if (performanceChart.options.scales.y.title) performanceChart.options.scales.y.title.text = getLangString('playertime.web.chart.tps');
            if (performanceChart.options.scales.y1.title) performanceChart.options.scales.y1.title.text = getLangString('playertime.web.chart.mspt');
            performanceChart.update();
        }
        if (diskChart) {
            diskChart.data.labels = [getLangString('playertime.web.chart.disk_used'), getLangString('playertime.web.chart.disk_free')];
            diskChart.update();
        }
    }

    // 获取语言字符串
    function getLangString(key, ...args) {
        const pattern = lang[key] || key;
        try {
            let result = pattern;
            args.forEach((arg, index) => {
                result = result.replace(new RegExp('\\{' + index + '\\}', 'g'), arg);
            });
            return result;
        } catch (e) {
            console.error(`格式化键的字符串失败: ${key}`, e);
            return pattern;
        }
    }


    // 切换主题
    function toggleTheme() {
        const currentTheme = document.documentElement.getAttribute('data-theme');
        const newTheme = currentTheme === 'light' ? 'dark' : 'light';
        document.documentElement.setAttribute('data-theme', newTheme);
        localStorage.setItem('theme', newTheme);
    }


    // 防呆
    function handleRefresh() {
        if (this.classList.contains('loading')) return;

        this.classList.add('loading');
        loadAllData().finally(() => {
            setTimeout(() => {
                this.classList.remove('loading');
            }, 500);
        });
    }


    async function loadAllData() {
        try {
            const [statsData, onlinePlayersData, playerCountsData, serverStatusData] = await Promise.all([
                fetchData('/api/stats'),
                fetchData('/api/online-players'),
                fetchData('/api/player-count'),
                fetchData('/api/server-status')
            ]);

            updateTable(statsData);
            updateOnlinePlayers(onlinePlayersData);
            updatePlayerCounts(playerCountsData);
            updateServerStatus(serverStatusData);

        } catch (error) {
            console.error('加载数据出错:', error);
            showError(getLangString('playertime.web.error.load_failed'));
        }
    }

    async function fetchData(url) {
        const response = await fetch(url);
        if (!response.ok) {
            const errorBody = await response.text().catch(() => '未知错误');
            throw new Error(`HTTP错误! 状态: ${response.status}, URL: ${url}, 内容: ${errorBody}`);
        }
        return response.json();
    }

    function updateTable(data) {
        if (!elements.statsTableBody || !elements.statsTableHeader) return;

        elements.statsTableBody.innerHTML = '';

        const headers = elements.statsTableHeader.querySelectorAll('th[data-lang-key]');
        headers.forEach(th => {
            const key = th.getAttribute('data-lang-key');
            if (lang[key]) {
                th.textContent = lang[key];
            }
        });

        if (!Array.isArray(data) || data.length === 0) {
            const row = elements.statsTableBody.insertRow();
            const cell = row.insertCell(0);
            cell.colSpan = 4;
            cell.textContent = getLangString('playertime.web.stats_table.empty');
            cell.style.textAlign = 'center';
            cell.style.fontStyle = 'italic';
            return;
        }

        data.forEach(player => {
            const row = elements.statsTableBody.insertRow();
            row.insertCell(0).textContent = player.playerName;
            row.insertCell(1).textContent = player.totalTimeFormatted;
            row.insertCell(2).textContent = player.last30DaysFormatted;
            row.insertCell(3).textContent = player.last7DaysFormatted;
        });
    }

    // 更新在线玩家列表
    function updateOnlinePlayers(data) {
        if (!elements.whitelistPlayers || !elements.nonWhitelistPlayers) return;

        const updateList = (element, players, emptyMessageKey) => {
            element.innerHTML = '';
            if (players?.length > 0) {
                players.forEach(player => {
                    const li = document.createElement('li');
                    li.innerHTML = `<span class="player-name">${player.name}</span>`;
                    element.appendChild(li);
                });
            } else {
                element.innerHTML = `<li>${getLangString(emptyMessageKey)}</li>`;
            }
        };

        updateList(elements.whitelistPlayers, data.whitelisted, 'playertime.web.status.online_list.empty_players');
        updateList(elements.nonWhitelistPlayers, data.non_whitelisted, 'playertime.web.status.online_list.empty_non_players');
    }

    // 更新在线玩家计数
    function updatePlayerCounts(data) {
        if (elements.totalCount) elements.totalCount.textContent = data.total || 0;
        if (elements.whitelistCount) elements.whitelistCount.textContent = data.whitelisted || 0;
        if (elements.nonWhitelistCount) elements.nonWhitelistCount.textContent = data.non_whitelisted || 0;

        document.getElementById('whitelist-count-badge').textContent = data.whitelisted || 0;
        document.getElementById('non-whitelist-count-badge').textContent = data.non_whitelisted || 0;
    }

    // 更新服务器状态信息和图表
    function updateServerStatus(data) {
        if (data.memory) {
            const usedMB = Math.round(data.memory.used / (1024 * 1024));
            const freeMB = Math.round(data.memory.free / (1024 * 1024));
            const percent = Math.round(data.memory.usage_percentage);

            if (memoryChart) {
                memoryChart.data.datasets[0].data = [usedMB, freeMB];
                memoryChart.data.labels = [getLangString('playertime.web.chart.memory_used'), getLangString('playertime.web.chart.memory_free')];
                memoryChart.update();
            }

            if (elements.memoryUsed) elements.memoryUsed.textContent = usedMB;
            if (elements.memoryFree) elements.memoryFree.textContent = freeMB;
            if (elements.memoryPercent) elements.memoryPercent.textContent = percent;
        }

        // 更新性能统计和图表（TPS/MSPT）
        if (data.server) {
            const mspt = data.server.average_tick_time_ms || 0;
            const tps = Math.min(20, 1000 / (mspt || 1)).toFixed(1);

            if (elements.tpsValue) elements.tpsValue.textContent = tps;
            if (elements.msptValue) elements.msptValue.textContent = mspt.toFixed(1);

            if (performanceChart) {
                tpsHistory.shift();
                tpsHistory.push(parseFloat(tps));
                msptHistory.shift();
                msptHistory.push(mspt);
                performanceChart.data.datasets[0].data = tpsHistory;
                performanceChart.data.datasets[1].data = msptHistory;

                performanceChart.update('none');

                setMetricColor(elements.tpsValue, parseFloat(tps), 18, 15, false);
                setMetricColor(elements.msptValue, mspt, 50, 60, true);
            }
        }

        // 更新磁盘统计和图表
        if (data.disk && data.disk.total > 0) {
            const totalGB = Math.round(data.disk.total / (1024 * 1024 * 1024));
            const usedGB = Math.round((data.disk.total - data.disk.free) / (1024 * 1024 * 1024));
            const freeGB = Math.round(data.disk.free / (1024 * 1024 * 1024));
            const percent = Math.round(data.disk.usage_percentage);

            if (diskChart) {
                diskChart.data.datasets[0].data = [usedGB, freeGB];
                diskChart.data.labels = [getLangString('playertime.web.chart.disk_used'), getLangString('playertime.web.chart.disk_free')];
                diskChart.update();
            }

            if (elements.diskTotal) elements.diskTotal.textContent = totalGB;
            if (elements.diskUsed) elements.diskUsed.textContent = usedGB;
            if (elements.diskFree) elements.diskFree.textContent = freeGB;
            if (elements.diskPercent) elements.diskPercent.textContent = percent;
        } else if (data.disk && data.disk.total === 0) {
            if (elements.diskTotal) elements.diskTotal.textContent = getLangString('playertime.web.server_status.not_available') || 'N/A';
            if (elements.diskUsed) elements.diskUsed.textContent = getLangString('playertime.web.server_status.not_available') || 'N/A';
            if (elements.diskFree) elements.diskFree.textContent = getLangString('playertime.web.server_status.not_available') || 'N/A';
            if (elements.diskPercent) elements.diskPercent.textContent = getLangString('playertime.web.server_status.not_available') || 'N/A';
            if (diskChart) {
                diskChart.data.datasets[0].data = [1, 0];
                diskChart.data.labels = [getLangString('playertime.web.server_status.not_available') || 'N/A', ''];
                diskChart.update();
            }
        }

        // 更新服务器信息
        if (elements.processors) elements.processors.textContent = data.available_processors || 0;
        if (data.server) {
            if (elements.serverVersion) elements.serverVersion.textContent = data.server.version || '未知';
            if (elements.serverPlayerCount) elements.serverPlayerCount.textContent = data.server.player_count || 0;
            if (elements.serverMaxPlayers) elements.serverMaxPlayers.textContent = data.server.max_players || 0;
            if (elements.serverMotd) elements.serverMotd.textContent = data.server.motd || '未知';
        }

        // 更新运行时间
        if (elements.uptime) elements.uptime.textContent = data.uptime_formatted || '加载中...';
    }


    function initCharts() {
        // 内存图表
        const memoryCtx = document.getElementById('memory-chart')?.getContext('2d');
        if (memoryCtx) {
            memoryChart = new Chart(memoryCtx, {
                type: 'doughnut',
                data: {
                    labels: [getLangString('playertime.web.chart.memory_used'), getLangString('playertime.web.chart.memory_free')],
                    datasets: [{
                        data: [0, 100],
                        backgroundColor: ['#4361ee', '#e9ecef'],
                        borderColor: [
                            getComputedStyle(document.documentElement).getPropertyValue('--card-bg').trim(),
                            getComputedStyle(document.documentElement).getPropertyValue('--card-bg').trim()
                        ],
                        borderWidth: 2
                    }]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    plugins: {
                        legend: {
                            position: 'top',
                            labels: {
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            }
                        },
                        tooltip: {
                            callbacks: {
                                label: function(context) {
                                    const label = context.label || '';
                                    const value = context.raw;
                                    return `${label}: ${value} MB`;
                                }
                            }
                        }
                    }
                }
            });
        }

        // 性能图表（TPS/MSPT）
        const perfCtx = document.getElementById('performance-chart')?.getContext('2d');
        if (perfCtx) {
            performanceChart = new Chart(perfCtx, {
                type: 'line',
                data: {
                    labels: performanceLabels,
                    datasets: [
                        {
                            label: getLangString('playertime.web.chart.tps'),
                            data: tpsHistory,
                            borderColor: '#4CAF50',
                            backgroundColor: 'rgba(76, 175, 80, 0.1)',
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 4,
                            tension: 0.3,
                            yAxisID: 'y'
                        },
                        {
                            label: getLangString('playertime.web.chart.mspt'),
                            data: msptHistory,
                            borderColor: '#FF5722',
                            backgroundColor: 'rgba(255, 87, 34, 0.1)',
                            borderWidth: 2,
                            pointRadius: 2,
                            pointHoverRadius: 4,
                            tension: 0.3,
                            yAxisID: 'y1'
                        }
                    ]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    animation: {
                        duration: 500
                    },
                    plugins: {
                        legend: {
                            position: 'top',
                            labels: {
                                boxWidth: 12,
                                padding: 20,
                                font: {
                                    size: 12
                                },
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            }
                        },
                        tooltip: {
                            mode: 'index',
                            intersect: false,
                            bodySpacing: 8,
                            callbacks: {
                                label: function(context) {
                                    let label = context.dataset.label || '';
                                    if (label) {
                                        label += ': ';
                                    }
                                    if (context.dataset.yAxisID === 'y') {
                                        label += context.raw.toFixed(1);
                                    } else {
                                        label += context.raw.toFixed(1) + ' ms';
                                    }
                                    return label;
                                }
                            }
                        }
                    },
                    scales: {
                        x: {
                            grid: {
                                display: false,
                                color: getComputedStyle(document.documentElement).getPropertyValue('--border-color').trim()
                            },
                            ticks: {
                                display: false
                            }
                        },
                        y: {
                            type: 'linear',
                            display: true,
                            position: 'left',
                            title: {
                                display: true,
                                text: getLangString('playertime.web.chart.tps'),
                                font: {
                                    weight: 'bold',
                                    color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                                },
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            },
                            min: 0,
                            max: 20,
                            grid: {
                                color: getComputedStyle(document.documentElement).getPropertyValue('--border-color').trim()
                            },
                            ticks: {
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            }
                        },
                        y1: {
                            type: 'linear',
                            display: true,
                            position: 'right',
                            title: {
                                display: true,
                                text: getLangString('playertime.web.chart.mspt'),
                                font: {
                                    weight: 'bold',
                                    color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                                },
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            },
                            min: 0,
                            suggestedMax: 100,
                            grid: {
                                drawOnChartArea: false,
                                color: getComputedStyle(document.documentElement).getPropertyValue('--border-color').trim()
                            },
                            ticks: {
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            }
                        }
                    }
                }
            });
        }

        // 磁盘图表
        const diskCtx = document.getElementById('disk-chart')?.getContext('2d');
        if (diskCtx) {
            diskChart = new Chart(diskCtx, {
                type: 'doughnut',
                data: {
                    labels: [getLangString('playertime.web.chart.disk_used'), getLangString('playertime.web.chart.disk_free')],
                    datasets: [{
                        data: [0, 100],
                        backgroundColor: ['#4361ee', '#e9ecef'],
                        borderColor: [
                            getComputedStyle(document.documentElement).getPropertyValue('--card-bg').trim(),
                            getComputedStyle(document.documentElement).getPropertyValue('--card-bg').trim()
                        ],
                        borderWidth: 2
                    }]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    plugins: {
                        legend: {
                            position: 'top',
                            labels: {
                                color: getComputedStyle(document.documentElement).getPropertyValue('--text-color').trim()
                            }
                        },
                        tooltip: {
                            callbacks: {
                                label: function(context) {
                                    const label = context.label || '';
                                    const value = context.raw;
                                    return `${label}: ${value} GB`;
                                }
                            }
                        }
                    }
                }
            });
        }
    }

    function showError(message) {
        const errorEl = document.createElement('div');
        errorEl.className = 'error-message';
        errorEl.textContent = message;
        document.body.appendChild(errorEl);

        setTimeout(() => {
            errorEl.classList.add('show');
            setTimeout(() => {
                errorEl.classList.remove('show');
                setTimeout(() => document.body.removeChild(errorEl), 300);
            }, 5000);
        }, 10);
    }

    function setMetricColor(element, value, goodThreshold, badThreshold, reverse = false) {
        if (!element) return;

        const numValue = parseFloat(value);
        let color = '';

        const successColor = getComputedStyle(document.documentElement).getPropertyValue('--success-color').trim();
        const warningColor = getComputedStyle(document.documentElement).getPropertyValue('--warning-color').trim();
        const dangerColor = getComputedStyle(document.documentElement).getPropertyValue('--danger-color').trim();

        if (reverse) { // 值越低越好（例如MSPT）
            if (numValue <= goodThreshold) {
                color = successColor;
            } else if (numValue <= badThreshold) {
                color = warningColor;
            } else {
                color = dangerColor;
            }
        } else { // 值越高越好（例如TPS）
            if (numValue >= goodThreshold) {
                color = successColor;
            } else if (numValue >= badThreshold) {
                color = warningColor;
            } else {
                color = dangerColor;
            }
        }

        element.style.color = color;
    }
});
